import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { VisualizationConfiguration } from '../../vis/common';
import { isEqual } from 'lodash-es';

export type VisStateSettings = {
  [id: string]: VisualizationConfiguration;
};
export type VisState = {
  activeVisualization?: string;
  openVisualizations: VisStateSettings;
};

export const initialState: VisState = {
  // activeVisualization: 'NodeLinkVis',
  openVisualizations: {},
};

export const visualizationSlice = createSlice({
  name: 'visualization',
  initialState,
  reducers: {
    removeVisualization: (state, action: PayloadAction<string>) => {
      if (state.openVisualizations[action.payload]) {
        delete state.openVisualizations[action.payload];
      }
    },
    setActiveVisualization: (state, action: PayloadAction<string>) => {
      state.activeVisualization = action.payload;
    },
    setVisualizationState: (state, action: PayloadAction<VisState>) => {
      if (action.payload.activeVisualization && !isEqual(action.payload, state)) {
        state.activeVisualization = action.payload.activeVisualization;
        state.openVisualizations = action.payload.openVisualizations || {};
      }
    },
    updateVisualization: (state, action: PayloadAction<{ id: string; settings: any }>) => {
      const { id, settings } = action.payload;
      state.openVisualizations[id] = settings;
    },
    reorderVisState: (state, action: PayloadAction<{ [id: string]: VisualizationConfiguration }>) => {
      state.openVisualizations = action.payload;
    },
  },
});

export const { removeVisualization, setActiveVisualization, setVisualizationState, updateVisualization, reorderVisState } =
  visualizationSlice.actions;

export const visualizationState = (state: RootState) => state.visualize;
export const visualizationAllSettings = (state: RootState) => state.visualize.openVisualizations;

export default visualizationSlice.reducer;
